using System;
using Gtk;
using Gdk;

public partial class MainWindow : Gtk.Window
{
	bool _IsDragging = false;

	public MainWindow () : base(Gtk.WindowType.Toplevel)
	{
		Build ();
		this.AddEvents( (int)(Gdk.EventMask.AllEventsMask) );
		
		// regist this window as drop target
/*		Gtk.Drag.DestSet( this,
		                 DestDefaults.All,
		                 new TargetEntry[1] { new TargetEntry("text/plain", TargetFlags.App, 1) },
						DragAction.Copy | DragAction.Move );
*/
		
		EnterNotifyEvent += delegate(object o, EnterNotifyEventArgs e) {
			Console.WriteLine("EnterNotifyEvent: ({0}, {1})", e.Event.X, e.Event.Y);
		};
		
		MotionNotifyEvent += delegate(object o, MotionNotifyEventArgs e) {
			Console.WriteLine("MotionNotifyEvent: ({0}, {1})", e.Event.X, e.Event.Y);
			if( _IsDragging )
			{
				TargetEntry entry = new TargetEntry( "text/plain", TargetFlags.App, 1 );
				Console.WriteLine("----BEGIN----");
				object ret = Gtk.Drag.Begin( this,
				           new TargetList(new TargetEntry[]{entry}),
				           DragAction.Copy,
				           1, e.Event
				           );
				Console.WriteLine("----END----");
			}
		};
		
		LeaveNotifyEvent += delegate(object o, LeaveNotifyEventArgs e) {
			Console.WriteLine("LeaveNotifyEvent: ({0}, {1})", e.Event.X, e.Event.Y);
		};
		
		ButtonPressEvent += delegate(object o, ButtonPressEventArgs e) {
			Console.WriteLine("ButtonPressEvent: ({0}, {1}) {2}", e.Event.X, e.Event.Y, e.Event.Type);
			 _IsDragging = true;
		};

		ButtonReleaseEvent += delegate(object o, ButtonReleaseEventArgs e) {
			Console.WriteLine("ButtonReleaseEvent: ({0}, {1}) {2}", e.Event.X, e.Event.Y, e.Event.Type);
			 _IsDragging = false;
		};

		DragBegin += delegate(object o, DragBeginArgs e) {
			Console.WriteLine("DragBegin:");
		};

		DragMotion += delegate(object o, DragMotionArgs e) {
			int width, height;
			
			this.GetSize( out width, out height );
			
			e.RetVal = ( e.Y < (height/2) );
			if( e.X < (width / 2) )
			{
				Gdk.Drag.Status( e.Context, DragAction.Move, e.Time );
				Console.WriteLine("DragMotion: move ({0})", e.RetVal);
			}
			else
			{
				Gdk.Drag.Status( e.Context, DragAction.Copy, e.Time );
				Console.WriteLine("DragMotion: copy ({0})", e.RetVal);
			}
		};
		
		
		DragLeave += delegate(object o, DragLeaveArgs e) {
			Console.WriteLine("DragLeave:");
		};
		
		DragDataReceived += delegate(object o, DragDataReceivedArgs e) {
			Console.WriteLine("DragDataReceived: info:{0}", e.Info);
			string text = e.SelectionData.Text;
			new Gtk.MessageDialog( this,
			                      DialogFlags.Modal,
			                      MessageType.Info,
			                      ButtonsType.Ok,
			                      text,
			                      null ).Run();
		};

		DragFailed += delegate(object o, DragFailedArgs e) {
			Console.WriteLine("DragFailed: {0}", e.DragResult);
		};

		DragDrop += delegate(object o, DragDropArgs e) {
			Console.WriteLine("DragDrop: {0}", e.Context.Action);
			e.RetVal = true;
		};

		DragEnd += delegate(object o, DragEndArgs e) {
			Console.WriteLine("DragEnd: {0}", e.Context.Action);
		};
	}

	protected void OnDeleteEvent (object sender, DeleteEventArgs a)
	{
		Application.Quit ();
		a.RetVal = true;
	}
}
